@@ -25,6 +25,7 @@ from __future__ import annotations
 import asyncio
 import pathlib
 import signal
+from typing import Any
 
 from . import log, payload
 from .events import EventGroup
@@ -94,7 +95,7 @@ class Brain:
         )
         self.ipc.events.include(_E)
 
-        self.stop_future: asyncio.Future | None = None
+        self.stop_future: asyncio.Future[None] | None = None
 
         self._waiting_for: tuple[int, int] | None = None
 
@@ -123,17 +124,24 @@ class Brain:
             starting the cluster and the smallest shard id
             of the cluster that should be starting.
         """
+
         if self._waiting_for is not None:
             server_uid, smallest_shard = self._waiting_for
-            if server_uid not in self.ipc.server_uids:
+            if (
+                server_uid not in self.ipc.server_uids
+                or smallest_shard in self.ipc.all_shards()
+            ):
+                # `server_uid not in self.ipc.server_uids`
                 # This means that the server that was supposed to start
                 # the cluster disconnected. This also means the
                 # cluster will never start, so we set _waiting_for
                 # to None.
+
+                # `smallest_shard in self.ipc.all_shards()`
+                # This means that the cluster has already started.
+
                 self._waiting_for = None
-            elif smallest_shard in self.ipc.all_shards():
-                # This means that the cluster has been started.
-                self._waiting_for = None
+
         return self._waiting_for
 
     @waiting_for.setter
@@ -143,7 +151,7 @@ class Brain:
     def run(self) -> None:
         """Run the brain, wait for the brain to stop, then cleanup."""
 
-        def sigstop(*args, **kwargs):
+        def sigstop(*args: Any, **kwargs: Any) -> None:
             self.stop()
 
         loop = asyncio.get_event_loop()
@@ -170,7 +178,7 @@ class Brain:
         await self.stop_future
 
     async def close(self) -> None:
-        """Shuts the brain down."""
+        """Shut the brain down."""
 
         self.server.stop()
         await self.server.close()
@@ -182,7 +190,7 @@ class Brain:
         await self.tasks.wait_for_all()
 
     def stop(self) -> None:
-        """Tells the brain to stop."""
+        """Tell the brain to stop."""
 
         assert self.stop_future
         self.stop_future.set_result(None)
@@ -191,7 +199,7 @@ class Brain:
         if len(self.ipc.server_uids) == 0:
             return None
 
-        if not all([c.ready for c in self.ipc.clusters.values()]):
+        if not all(c.ready for c in self.ipc.clusters.values()):
             return None
 
         all_shard_ids = self.ipc.all_shards()
@@ -205,12 +213,11 @@ class Brain:
         else:
             return None
 
-        shards_to_launch = set(range(0, self.total_shards))
-        shards_to_launch.difference_update(all_shard_ids)
-        if len(shards_to_launch) == 0:
+        shards_to_launch = set(range(self.total_shards)) - all_shard_ids
+        if not shards_to_launch:
             return None
 
-        return s.uid, list(shards_to_launch)[0 : self.shards_per_cluster]
+        return s.uid, list(shards_to_launch)[: self.shards_per_cluster]
 
     async def _send_brain_uid_loop(self) -> None:
         while True:
