@@ -36,12 +36,13 @@ from .callbacks import CallbackHandler, NoResponse
 from .commands import CommandHandler
 from .events import EventGroup, EventHandler
 from .info_classes import ClusterInfo, ServerInfo
+from .ipc_base import IpcBase
 from .task_manager import TaskManager
 
 __all__ = ("IpcClient",)
 
 
-class IpcClient:
+class IpcClient(IpcBase):
     """A connection to a :class:`~ipc_server.IpcServer`.
 
     Parameters
@@ -137,11 +138,11 @@ class IpcClient:
 
         all_shards: set[int] = set()
         for c in self.clusters.values():
-            if not c.ready:
-                continue
-            if c.server_uid not in self.servers:
-                continue
-            if c.uid not in self.servers[c.server_uid].cluster_uids:
+            if not (
+                c.ready
+                and c.server_uid in self.servers
+                and c.uid in self.servers[c.server_uid].cluster_uids
+            ):
                 continue
             all_shards.update(c.shard_ids)
         return all_shards
@@ -186,32 +187,6 @@ class IpcClient:
 
         self.tasks.create_task(self._start()).add_done_callback(_stop)
 
-    def stop(self) -> None:
-        """Tell the client to stop."""
-
-        if self.stop_future is not None and not self.stop_future.done():
-            self.stop_future.set_result(None)
-        if self.ready_future is not None and not self.ready_future.done():
-            self.ready_future.cancel()
-
-    async def close(self) -> None:
-        """Disconnect and close the client."""
-
-        self.tasks.cancel_all()
-        await self.tasks.wait_for_all()
-
-    async def wait_until_ready(self) -> None:
-        """Wait until the client is either ready or shutting down."""
-
-        assert self.ready_future is not None
-        await self.ready_future
-
-    async def join(self) -> None:
-        """Wait until the client is shutting down."""
-
-        assert self.stop_future is not None
-        await self.stop_future
-
     async def send_not_found_response(
         self, to: Iterable[int], callback: int
     ) -> None:
@@ -329,7 +304,7 @@ class IpcClient:
                 pass
             except ConnectionClosed as e:
                 if e.code == close_codes.INVALID_TOKEN:
-                    raise exceptions.InvalidIpcToken
+                    raise exceptions.InvalidIpcToken from e
             except asyncio.CancelledError:
                 reconnect = False
             finally:
@@ -356,8 +331,7 @@ class IpcClient:
         self.logger.debug(f"Handshake successful, uid {self.uid}")
 
     def _update_clients(self, client_uids: set[int]) -> None:
-        disconnected = self.client_uids.difference(client_uids)
-        if disconnected:
+        if self.client_uids.difference(client_uids):
             self.callbacks.handle_disconnects()
 
         self.client_uids = client_uids
@@ -365,18 +339,11 @@ class IpcClient:
             if sid not in self.client_uids:
                 del self.servers[sid]
 
-        to_remove: list[int] = []  # a list of cluster ids to remove
-
         for cid, c in list(self.clusters.items()):
-            if cid not in self.client_uids:
-                to_remove.append(c.cluster_id)
-                del self.clusters[cid]
-
-        for cid in to_remove:
-            if cid in self.clusters_by_cluster_id:
-                del self.clusters_by_cluster_id[cid]
-            else:
-                self.logger.warn("Something's up with clusters_by_cluster_id.")
+            if cid in self.client_uids:
+                continue
+            del self.clusters[cid]
+            del self.clusters_by_cluster_id[c.cluster_id]
 
     async def _recv_loop(self, ws: client.WebSocketClientProtocol) -> None:
         async for msg in ws:
