@@ -24,7 +24,7 @@ from __future__ import annotations
 
 import asyncio
 import traceback
-from typing import Any, Awaitable, Iterable, Type, TypeVar
+from typing import Any, Coroutine, Generator, Iterable, Type, TypeVar
 
 from . import log
 
@@ -61,7 +61,7 @@ class TaskManager:
 
     def create_task(
         self,
-        coro: Awaitable[_T],
+        coro: Generator[Any, None, _T] | Coroutine[Any, None, _T],
         *,
         name: str | None = None,
         ignored_exceptions: Iterable[Type[Exception]] = tuple(),
@@ -104,8 +104,7 @@ class TaskManager:
                     self.logger.error("Exception in task callback:")
                     self.logger.error(traceback.format_exc())
             finally:
-                if tid in self._tasks:
-                    del self._tasks[tid]
+                self._tasks.pop(tid, None)
 
         t = asyncio.create_task(coro, name=name)
         self._tasks[tid] = _TaskWrapper(allow_cancel, allow_wait, t)
@@ -129,12 +128,9 @@ class TaskManager:
             How long to wait for a task before giving up.
         """
 
-        to_wait: list[asyncio.Task[Any]] = []
-        for t in self._tasks.values():
-            if t.allow_wait:
-                to_wait.append(t.t)
+        to_wait = [t.t for t in self._tasks.values() if t.allow_wait]
 
-        if len(to_wait) == 0:
+        if not to_wait:
             return
 
         await asyncio.wait(to_wait, timeout=timeout)
